@@ -8,7 +8,10 @@
 namespace rocksdb {
 
 namespace {
-inline double SafeDivide(double a, double b) { return b == 0.0 ? 0 : a / b; }
+template<class T>
+inline T SafeDivide(T a, T b) {
+  return b == 0 ? 0 : a / b;
+}
 }  // namespace
 
 void EventHelpers::AppendCurrentTime(JSONWriter* jwriter) {
@@ -53,7 +56,7 @@ void EventHelpers::LogAndNotifyTableFileCreation(
                 info.table_properties.filter_policy_name;
 
     // user collected properties
-    for (const auto& prop : info.table_properties.user_collected_properties) {
+    for (const auto& prop : info.table_properties.readable_properties) {
       jwriter << prop.first << prop.second;
     }
     jwriter.EndObject();
