@@ -31,14 +31,21 @@ void IOStatsContext::Reset() {
   logger_nanos = 0;
 }
 
-#define OUTPUT(counter) #counter << " = " << counter << ", "
+#define IOSTATS_CONTEXT_OUTPUT(counter) #counter << " = " << counter << ", "
 
 std::string IOStatsContext::ToString() const {
   std::ostringstream ss;
-  ss << OUTPUT(thread_pool_id) << OUTPUT(bytes_read) << OUTPUT(bytes_written)
-     << OUTPUT(open_nanos) << OUTPUT(allocate_nanos) << OUTPUT(write_nanos)
-     << OUTPUT(read_nanos) << OUTPUT(range_sync_nanos) << OUTPUT(fsync_nanos)
-     << OUTPUT(prepare_write_nanos) << OUTPUT(logger_nanos);
+  ss << IOSTATS_CONTEXT_OUTPUT(thread_pool_id)
+     << IOSTATS_CONTEXT_OUTPUT(bytes_read)
+     << IOSTATS_CONTEXT_OUTPUT(bytes_written)
+     << IOSTATS_CONTEXT_OUTPUT(open_nanos)
+     << IOSTATS_CONTEXT_OUTPUT(allocate_nanos)
+     << IOSTATS_CONTEXT_OUTPUT(write_nanos)
+     << IOSTATS_CONTEXT_OUTPUT(read_nanos)
+     << IOSTATS_CONTEXT_OUTPUT(range_sync_nanos)
+     << IOSTATS_CONTEXT_OUTPUT(fsync_nanos)
+     << IOSTATS_CONTEXT_OUTPUT(prepare_write_nanos)
+     << IOSTATS_CONTEXT_OUTPUT(logger_nanos);
 
   return ss.str();
 }
